#!/usr/bin/env python
##################################################
# Gnuradio Python Flow Graph
# Title: Pfb Decim Perf
#
# Authored by Tom Rondeau
# Feb. 27, 2014
##################################################

from gnuradio import analog
from gnuradio import blocks
from gnuradio import eng_notation
from gnuradio import filter
from gnuradio import gr
from gnuradio.ctrlport.monitor import *
from gnuradio.eng_option import eng_option
from gnuradio.filter import firdes
from gnuradio.filter import pfb
from optparse import OptionParser
import scipy

class pfb_decim_perf(gr.top_block):

    def __init__(self, decim, channel, ntaps, nsamps):
        gr.top_block.__init__(self, "Pfb Decim Perf")

        ##################################################
        # Variables
        ##################################################
        self.decim = decim
        self.ntaps = ntaps
        self.taps = taps = scipy.random.random(ntaps)
        self.samp_rate = samp_rate = 100000
        self.nsamps = nsamps

        chan_spacing = samp_rate / float(decim)
        f_chan = chan_spacing * channel
        if(f_chan > samp_rate/2):
            f_chan -= samp_rate

        ##################################################
        # Blocks
        ##################################################
        # FIR filters / EXP rotator
        self.pfb_decimator_ccf_3 = pfb.decimator_ccf(
        	  decim,
        	  (taps),
        	  0,
        	  100,
                  False,
                  False)

        # FIR filters / FFT rotator
        self.pfb_decimator_ccf_2 = pfb.decimator_ccf(
        	  decim,
        	  (taps),
        	  0,
        	  100,
                  True,
                  False)

        # FFT filters / EXP rotator
        self.pfb_decimator_ccf_1 = pfb.decimator_ccf(
        	  decim,
        	  (taps),
        	  0,
        	  100,
                  False,
                  True)

        # FFT filters / FFT rotator
        self.pfb_decimator_ccf_0 = pfb.decimator_ccf(
        	  decim,
        	  (taps),
        	  0,
        	  100,
                  True,
                  True)

        self.fir_filter_xxx_0 = filter.fir_filter_ccc(decim, (taps))
        self.fir_filter_xxx_0.declare_sample_delay(0)
        self.fft_filter_xxx_0 = filter.fft_filter_ccc(decim, (taps), 1)
        self.fft_filter_xxx_0.declare_sample_delay(0)
        self.blocks_head_0 = blocks.head(gr.sizeof_gr_complex*1, int(nsamps))
        self.blocks_null_sink_0 = blocks.null_sink(gr.sizeof_gr_complex*1)
        self.analog_sig_source_x_0 = analog.sig_source_c(samp_rate, analog.GR_COS_WAVE, 1000, 1, 0)

        self.fir_chan_nco = analog.sig_source_c(samp_rate, analog.GR_COS_WAVE, f_chan, 1, 0)
        self.fir_chan_mix = blocks.multiply_cc()
        self.fft_chan_nco = analog.sig_source_c(samp_rate, analog.GR_COS_WAVE, f_chan, 1, 0)
        self.fft_chan_mix = blocks.multiply_cc()

        self.xlating_fir_0 = filter.freq_xlating_fir_filter_ccc(decim, (taps), f_chan, samp_rate)

        ##################################################
        # Connections
        ##################################################
        self.connect((self.blocks_head_0, 0), (self.pfb_decimator_ccf_0, 0))
        self.connect((self.analog_sig_source_x_0, 0), (self.blocks_head_0, 0))
        self.connect((self.blocks_head_0, 0), (self.pfb_decimator_ccf_1, 0))
        self.connect((self.blocks_head_0, 0), (self.pfb_decimator_ccf_2, 0))
        self.connect((self.blocks_head_0, 0), (self.pfb_decimator_ccf_3, 0))
        self.connect((self.pfb_decimator_ccf_3, 0), (self.blocks_null_sink_0, 5))
        self.connect((self.pfb_decimator_ccf_0, 0), (self.blocks_null_sink_0, 2))
        self.connect((self.pfb_decimator_ccf_2, 0), (self.blocks_null_sink_0, 4))
        self.connect((self.pfb_decimator_ccf_1, 0), (self.blocks_null_sink_0, 3))

        self.connect((self.blocks_head_0, 0), (self.fir_chan_mix, 0))
        self.connect((self.fir_chan_nco, 0), (self.fir_chan_mix, 1))
        self.connect((self.fir_chan_mix, 0), (self.fir_filter_xxx_0, 0))
        self.connect((self.fir_filter_xxx_0, 0), (self.blocks_null_sink_0, 0))

        self.connect((self.blocks_head_0, 0), (self.fft_chan_mix, 0))
        self.connect((self.fft_chan_nco, 0), (self.fft_chan_mix, 1))
        self.connect((self.fft_chan_mix, 0), (self.fft_filter_xxx_0, 0))
        self.connect((self.fft_filter_xxx_0, 0), (self.blocks_null_sink_0, 1))

        self.connect((self.blocks_head_0, 0), (self.xlating_fir_0, 0))
        self.connect((self.xlating_fir_0, 0), (self.blocks_null_sink_0, 6))


    def get_decim(self):
        return self.decim

    def set_decim(self, decim):
        self.decim = decim
        self.set_taps(firdes.low_pass_2(1, self.decim, 0.1, 0.01, 60))

    def get_taps(self):
        return self.taps

    def set_taps(self, taps):
        self.taps = taps
        self.set_ntaps(len(self.taps))
        self.fft_filter_xxx_0.set_taps((self.taps))
        self.pfb_decimator_ccf_0.set_taps((self.taps))
        self.fir_filter_xxx_0.set_taps((self.taps))
        self.pfb_decimator_ccf_1.set_taps((self.taps))
        self.pfb_decimator_ccf_2.set_taps((self.taps))
        self.pfb_decimator_ccf_3.set_taps((self.taps))
        self.xlating_fir_0.set_taps((self.taps))

    def get_samp_rate(self):
        return self.samp_rate

    def set_samp_rate(self, samp_rate):
        self.samp_rate = samp_rate
        self.analog_sig_source_x_0.set_sampling_freq(self.samp_rate)

    def get_ntaps(self):
        return self.ntaps

    def set_ntaps(self, ntaps):
        self.ntaps = ntaps

if __name__ == '__main__':
    parser = OptionParser(option_class=eng_option, usage="%prog: [options]")
    (options, args) = parser.parse_args()

    print("Decim,ntaps,FIR,FFT,PFB FIR/Exp, PFB FIR/FFT, PFB FFT/Exp, PFB FFT/FFT, Xlating")

    channel = 2
    iters = 20
    nsamps = 1e5
    tps = scipy.float64(gr.high_res_timer_tps())

    decim_start = 4
    decim_stop = 64
    decim_skip = 4
    tpc_start = 8
    tpc_stop = 64
    tpc_skip = 4

    for decim in xrange(decim_start, decim_stop+1, decim_skip):
        for ntaps in xrange(tpc_start, tpc_stop+1, tpc_skip):

            fir_times = scipy.array(iters*[0,], dtype=scipy.float64)
            fft_times = scipy.array(iters*[0,], dtype=scipy.float64)
            pfb0_times = scipy.array(iters*[0,], dtype=scipy.float64)
            pfb1_times = scipy.array(iters*[0,], dtype=scipy.float64)
            pfb2_times = scipy.array(iters*[0,], dtype=scipy.float64)
            pfb3_times = scipy.array(iters*[0,], dtype=scipy.float64)
            xlate_times = scipy.array(iters*[0,], dtype=scipy.float64)

            for i in xrange(iters):
                tb = pfb_decim_perf(decim, channel, decim*ntaps, nsamps)
                tb.start()
                tb.wait()

                fir_times[i] = (tb.fir_filter_xxx_0.pc_work_time_total()/tps +
                                tb.fir_chan_nco.pc_work_time_total()/tps +
                                tb.fir_chan_mix.pc_work_time_total()/tps)
                fft_times[i] = (tb.fft_filter_xxx_0.pc_work_time_total()/tps +
                                tb.fir_chan_nco.pc_work_time_total()/tps +
                                tb.fir_chan_mix.pc_work_time_total()/tps)
                pfb0_times[i] = (tb.pfb_decimator_ccf_3.pfb.pc_work_time_total()/tps)
                pfb1_times[i] = (tb.pfb_decimator_ccf_2.pfb.pc_work_time_total()/tps)
                pfb2_times[i] = (tb.pfb_decimator_ccf_1.pfb.pc_work_time_total()/tps)
                pfb3_times[i] = (tb.pfb_decimator_ccf_0.pfb.pc_work_time_total()/tps)
                xlate_times[i] = (tb.xlating_fir_0.pc_work_time_total()/tps)

                del tb

            print("{0},{1},{2},{3},{4},{5},{6},{7},{8}".format(
                    decim, ntaps,
                    fir_times.mean(),
                    fft_times.mean(),
                    pfb0_times.mean(),
                    pfb1_times.mean(),
                    pfb2_times.mean(),
                    pfb3_times.mean(),
                    xlate_times.mean()))

